﻿// --------------------------------
// <copyright file="Building.cs" company="COMPANY">
//     Copyright (C) COMPANY.  All rights reserved.
// </copyright>
// --------------------------------

using UnityEngine;

/// <summary>
/// Building class.
/// </summary>
[RequireComponent(typeof(Renderer))]
public class Building : MonoBehaviour
{
	private const int PeoplePerUnitFloorspace = 50;

	/// <summary>
	/// Population manager, set by parent.
	/// </summary>
	public PopulationManager populationManager;

	/// <summary>
	/// Population of self.
	/// </summary>
	public int population;

	/// <summary>
	/// Whether the building is intact or destroyed.
	/// </summary>
	public bool isIntact = true;

	/// <summary>
	/// Position at beginning of simulation.
	/// </summary>
	private Vector2 homePosition;

    /// <summary>
    /// Initialize script state.
    /// </summary>
    internal void Start()
    {
		var floorSize = this.transform.localScale.x * this.transform.localScale.z;
		var floors = (int)(this.transform.localScale.y / Constants.floorHeight);
		var floorSpace = floorSize * floors;
		this.population = (int)(floorSpace * PeoplePerUnitFloorspace);
        //Debug.Log("with a floor size of " + floorSize + " and " + floors + " floors, floorspace = " + floorSpace +
        //          " and population = " + this.population);

		this.populationManager.population += this.population;

		this.homePosition = new Vector2(this.transform.localPosition.x, this.transform.localPosition.z);
    }

	/// <summary>
	/// Update script, called on a fixed interval.
	/// </summary>
	internal void FixedUpdate()
	{
		if (!this.isIntact)
		{
            if (this.transform.position.y < -10f)
            {
                Destroy(this.gameObject);
            }

			return;
		}

		var distance = Vector2.Distance(this.homePosition, new Vector2(this.transform.localPosition.x, this.transform.localPosition.z));
		if (distance > 0.02f)
		{
			Debug.Log("high building movement " + distance + ", collapse");
			this.Collapse();
		}

		var rotation = Quaternion.Angle(Quaternion.identity, this.transform.localRotation);
		if (rotation > 2f)
		{
			Debug.Log("high building rotation " + rotation + ", collapse");
			this.Collapse();
		}
	}

	/// <summary>
	/// Called when this collider/rigidbody has begun touching another rigidbody/collider.
	/// </summary>
	/// <param name="collision">Collision details.</param>
	internal void OnCollisionEnter(Collision collision)
	{
		if (!this.isIntact)
		{
			return;
		}

		if (collision.relativeVelocity.magnitude > 1f)
		{
			Debug.Log("high building forces " + collision.relativeVelocity.magnitude + ", collapse");
			this.Collapse();
		}
	}

	/// <summary>
	/// Destroy this building.
	/// </summary>
	private void Collapse()
	{
		this.isIntact = false;

	    this.populationManager.casualties += this.population;

        this.GetComponent<Renderer>().material.color = Color.red;
        
        Destroy(this.GetComponent<Collider>());

	    var particleEffectManager = FindObjectOfType<ParticleEffectManager>();
        if (particleEffectManager)
        {
            particleEffectManager.InstantiateParticles(particleEffectManager.dustPrefab, new Vector3(this.transform.position.x, 0f, this.transform.position.z));
        }
	}
}