﻿// --------------------------------
// <copyright file="Tornado.cs" company="COMPANY">
//     Copyright (C) COMPANY.  All rights reserved.
// </copyright>
// --------------------------------

using System.Collections;
using UnityEngine;

/// <summary>
/// Tornado class.
/// </summary>
public class Tornado : MonoBehaviour
{
	/// <summary>
	/// Target position.
	/// </summary>
	public Vector3 target;

	/// <summary>
	/// Rate of movement.
	/// </summary>
	public float speed = 0.25f;

	/// <summary>
	/// Destructive power.
	/// </summary>
	public float power = 10f;

	/// <summary>
	/// Input manager.
	/// </summary>
	private InputManager input;

    /// <summary>
    /// Initialize script state.
    /// </summary>
    internal void Start()
    {
		this.target = this.transform.position;

		var particleSystem = this.GetComponentInChildren<ParticleSystem>();
		this.StartCoroutine(this.Kill(particleSystem.duration + particleSystem.startLifetime));
    }

	/// <summary>
	/// Called when script is enabled.
	/// </summary>
	internal void OnEnable()
	{
		if (!this.input)
		{
			this.input = FindObjectOfType<InputManager>();
		}

		this.input.PickEvent += this.HandlePickEvent;
	}
	
	/// <summary>
	/// Called when script is disabled.
	/// </summary>
	internal void OnDisable()
	{
		if (this.input)
		{
			this.input.PickEvent -= this.HandlePickEvent;
		}
	}

	/// <summary>
	/// Update script, called on a fixed interval.
	/// </summary>
	internal void FixedUpdate()
	{
		// move toward target
		if (Vector3.Distance(this.rigidbody.position, this.target) > float.Epsilon)
		{
			var direction = (this.target - this.rigidbody.position);
			this.rigidbody.MovePosition(this.rigidbody.position + (direction * Time.deltaTime * this.speed));
		}
    }

	/// <summary>
	/// Called once per frame for every other Collider that is touching the trigger.
	/// </summary>
	/// <param name="other">Collider details.</param>
	internal void OnTriggerStay(Collider other)
	{
		Debug.Log("tornado impacting " + other.gameObject.name);
		if (other.attachedRigidbody)
		{
			Debug.Log("tornado impacting " + other.gameObject.name);
			//var impactPoint = other.ClosestPointOnBounds(this.transform.position);
			//var direction = impactPoint - this.transform.position;
			//var windDirection = Vector3.Cross(Vector3.down, direction.normalized);
			//Debug.Log("tornado impacting " + other.gameObject.name + " at " + impactPoint + ", " + direction + ", " + windDirection);
			//other.rigidbody.AddForceAtPosition(windDirection, impactPoint);

			other.attachedRigidbody.AddTorque(Vector3.up * this.power);
		}
	}

	private void HandlePickEvent(Vector3 point)
	{
		this.target = new Vector3(point.x, 0f, point.z);
	}

	/// <summary>
	/// Kill this after particles expire.
	/// </summary>
	/// <returns>Coroutine iterator.</returns>
	internal IEnumerator Kill(float lifespan)
	{
		yield return new WaitForSeconds(lifespan);

		var particles = this.GetComponentInChildren<ParticleManager>();
		if (particles)
		{
			particles.Dismiss();
		}

		Destroy(this.gameObject);
	}
}